"""Keeper templates module."""
#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

from .access_secret_store_template import EscrowAccessSecretStoreTemplate
from .template_manager import TemplateStoreManager
